#ifndef FastImageGen_H
#define FastImageGen_H
/*########################################################################
	File:			FastImageGen.h
	Creation date:	6th June 2002

	Copyright 2002 Mark Williams, all rights reserved.
	Contact: mark@extrabit.com
	http://www.extrabit.com

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

########################################################################*/

/*########################################################################
	Headers
########################################################################*/

#include "rgbcolour.h"
#include "Vector2.h"
#include "ImageGen.h"

/*########################################################################
	Declarations
########################################################################*/

/*########################################################################
	Class Definition
########################################################################*/

/**	This class processes a FractalImage by calculating it pixel by pixel.
 *	This is much faster than the technique used by ProgressiveFractalImage,
 *	but it does mean there is nothing to see until the whole image has
 *	been calculated.
 */
class FastImageGen : public ImageGen
{
public:
	FastImageGen (FractalImage*) ;

	// Overriden from ImageGen base class.
	void init (void) ;
	void restart (void) ;
	bool process (void) ;
	bool inq_done (void) const ;
	int inq_progress (void) const ;

private:
	/**	FractalImage. */
	FractalImage* m_image ;

	/** Flag that is set once processing is finished. */
	bool m_done ;

	/** Progress counter - increments from 0 to 100 during processing. */
	int m_progress ;

	/**	'x' offset to current pixel. */
	int m_x ;

	/**	'y' offset to current pixel. */
	int m_y ;

	/**	Pointer to current RGB pixel. */
	rgbcolour_t* m_curr_rgb_pixel ;

	/**	Pointer to current pixel iteration count, */
	unsigned short* m_curr_pixel_itns ;

	/**	Position of bottom left corner of current pixel. */
	Vector2 m_curr_pixel ;

	/**	Position in fractal space of first pixel on the current row. */
	Vector2 m_rowstart ;

	/**	Offset in fractal space from a pixel to the next.
	 *	This is determined from the view transform and the image size.
	 */
	Vector2 m_xoffs ;

	/**	Offset in fractal space from a pixel in one row to the next.
	 *	This is determined from the view transform and the image size.
	 */
	Vector2 m_yoffs ;
} ;


/*########################################################################
	End
########################################################################*/
#endif
