/*########################################################################
	MainFrm.cpp

	Copyright 2002 Mark Williams, all rights reserved.
	Contact: mark@extrabit.com
	http://www.extrabit.com

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

########################################################################*/

#include "stdafx.h"
#include "FractalViewer.h"

#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	ON_WM_CREATE()
	ON_WM_SETFOCUS()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CENTRE,
	ID_INDICATOR_WIDTH,
	ID_INDICATOR_ANGLE,
};

// Set sizes of status bar fields.
static const int SBWIDTH_CENTRE = 150 ;
static const int SBWIDTH_WIDTH  = 120 ;
static const int SBWIDTH_ANGLE  = 120 ;


/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

/////////////////////////////////////////////////////////////////////////////
CMainFrame::CMainFrame()
{
}


/////////////////////////////////////////////////////////////////////////////
CMainFrame::~CMainFrame()
{
}


/////////////////////////////////////////////////////////////////////////////
void CMainFrame::update_frac_wnd (void)
{
	m_wndView.Invalidate (FALSE) ;
	m_wndView.UpdateWindow() ;
}


/////////////////////////////////////////////////////////////////////////////
void CMainFrame::update_status_bar (double x, double y, double width, double angle)
{
	CString msg ;

	msg.Format (IDS_SHOW_CENTRE, x, y) ;
	m_wndStatusBar.SetPaneText (1, msg) ;

	msg.Format (IDS_SHOW_WIDTH, width) ;
	m_wndStatusBar.SetPaneText (2, msg) ;

	msg.Format (IDS_SHOW_ANGLE, angle) ;
	m_wndStatusBar.SetPaneText (3, msg) ;
}


/////////////////////////////////////////////////////////////////////////////
void CMainFrame::show_status (CString msg)
{
	m_wndStatusBar.SetPaneText (0, msg) ;
}


/////////////////////////////////////////////////////////////////////////////
int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	// create a view to occupy the client area of the frame
	if (!m_wndView.Create(NULL, NULL, AFX_WS_DEFAULT_VIEW,
		CRect(0, 0, 0, 0), this, AFX_IDW_PANE_FIRST, NULL))
	{
		TRACE0("Failed to create view window\n");
		return -1;
	}

	FractalImage* image = ((CFractalViewerApp*)AfxGetApp())->inq_image() ;
	m_wndView.set_image (image) ;

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}
	m_wndStatusBar.SetPaneInfo (1, ID_INDICATOR_CENTRE, SBPS_NORMAL, SBWIDTH_CENTRE) ;
	m_wndStatusBar.SetPaneInfo (2, ID_INDICATOR_WIDTH,  SBPS_NORMAL, SBWIDTH_WIDTH) ;
	m_wndStatusBar.SetPaneInfo (3, ID_INDICATOR_ANGLE,  SBPS_NORMAL, SBWIDTH_ANGLE) ;

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;

	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
	cs.lpszClass = AfxRegisterWndClass(0, 0, 0, AfxGetApp()->LoadIcon(IDR_MAINFRAME)) ;

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers
void CMainFrame::OnSetFocus(CWnd* /* pOldWnd */)
{
	// forward focus to the view window
	m_wndView.SetFocus();
}


/////////////////////////////////////////////////////////////////////////////
BOOL CMainFrame::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
	// let the view have first crack at the command
	if (m_wndView.OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;

	// otherwise, do default handling
	return CFrameWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

