/*########################################################################
	File:			Z2PlusZProcessor.cpp
	Author:			Mark
	Creation date:	25th May 2002

	Copyright 2002 Mark Williams.

########################################################################*/

/*########################################################################
	Headers
########################################################################*/

#include "StdAfx.h"

#include "ComplexNumber.h"
#include "Z2PlusZProcessor.h"

/*########################################################################
	Local declarations
########################################################################*/

/*########################################################################
	Local definitions
########################################################################*/


/**	Performs one iteration in the Z2PlusZProcessor calculation.
 */
ComplexNumber Z2PlusZProcessor::iterate (const ComplexNumber& z, const ComplexNumber& u) const
{
	return z.inq_square() + z + u ;
}


/**	Calculates the number of iterations needed to determine that
 *	the given point is outside the set, or returns 'max_itns'
 *	if the point may be in the set.
 */
int Z2PlusZProcessor::iterate_to_end (const ComplexNumber& initial_z, const ComplexNumber& v, int max_itns) const
{
	// Use local variables to help optimiser,
	ComplexNumber u = v ;
	ComplexNumber z = initial_z ;

	for (int i = 0 ; i < max_itns ; i++)
	{
		z = z.inq_square() + z + u ;
		if (z.inq_magnitude2() > 4.0)
		{
			break ;
		}
	}

	return i ;
}


/*########################################################################
	End
########################################################################*/
