/*########################################################################
	File:			Z4Processor.cpp
	Creation date:	6th June 2002

	Copyright 2002 Mark Williams, all rights reserved.
	Contact: mark@extrabit.com
	http://www.extrabit.com

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

########################################################################*/

/*########################################################################
	Headers
########################################################################*/

#include "StdAfx.h"

#include "ComplexNumber.h"
#include "Z4Processor.h"

/*########################################################################
	Local declarations
########################################################################*/

/*########################################################################
	Local definitions
########################################################################*/


/**	Performs one iteration in the Z4Processor calculation.
 */
ComplexNumber Z4Processor::iterate (const ComplexNumber& z, const ComplexNumber& u) const
{
	return z.inq_square().inq_square() + u ;
}


/**	Calculates the number of iterations needed to determine that
 *	the given point is outside the set, or returns 'max_itns'
 *	if the point may be in the set.
 */
unsigned short Z4Processor::iterate_to_end (const ComplexNumber& initial_z, const ComplexNumber& v, unsigned short max_itns) const
{
	// Use local variables to help optimiser,
	ComplexNumber u = v ;
	ComplexNumber z = initial_z ;

	for (unsigned short i = 0 ; i < max_itns ; i++)
	{
		z = z.inq_square().inq_square() + u ;
		if (z.inq_magnitude2() > 4.0)
		{
			break ;
		}
	}

	return i ;
}


/*########################################################################
	End
########################################################################*/
